/* ********************************************************************

Parts of this file are taken from Bob Trower's implementation of base64.

LICENCE:        Copyright (c) 2001 Bob Trower, Trantor Standard Systems Inc.

                Permission is hereby granted, free of charge, to any person
                obtaining a copy of this software and associated
                documentation files (the "Software"), to deal in the
                Software without restriction, including without limitation
                the rights to use, copy, modify, merge, publish, distribute,
                sublicense, and/or sell copies of the Software, and to
                permit persons to whom the Software is furnished to do so,
                subject to the following conditions:

                The above copyright notice and this permission notice shall
                be included in all copies or substantial portions of the
                Software.

                THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
                KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
                WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
                PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
                OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
                OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
                OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
                SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


*********************************************************************** */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "b64.h"

int base64encode(char *result, char *data, int lenData){

     static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
     /* static const char cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq"; */

     void encodeblock( unsigned char *in, unsigned char *out, int len )
     {
         out[0] = (unsigned char) cb64[ (int)(in[0] >> 2) ];
         out[1] = (unsigned char) cb64[ (int)(((in[0] & 0x03) << 4) | ((in[1] & 0xf0) >> 4)) ];
         out[2] = (unsigned char) (len > 1 ? cb64[ (int)(((in[1] & 0x0f) << 2) | ((in[2] & 0xc0) >> 6)) ] : '=');
         out[3] = (unsigned char) (len > 2 ? cb64[ (int)(in[2] & 0x3f) ] : '=');
     }

     int i=0;
     int pos=0;
     char iblock[3];
     char oblock[4];
     while (i < lenData) {
          iblock[0] = data[i];
	  iblock[1] = data[i+1];
	  iblock[2] = data[i+2];
	  /*
	  puts(iblock);
	  puts(oblock);
	  */
	  encodeblock(iblock,oblock,lenData-i);
	  result[pos] = oblock[0];
	  result[pos+1] = oblock[1];
	  result[pos+2] = oblock[2];
	  result[pos+3] = oblock[3];
	  i +=3;
	  pos += 4;
     }
     /* result[pos] = 0; */
     return pos;

}

int base64decode(char *result, char *data){

     /* static const char cb64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"; */
     static const char cd64[]="|$$$}rstuvwxyz{$$$$$$$>?@ABCDEFGHIJKLMNOPQRSTUVW$$$$$$XYZ[\\]^_`abcdefghijklmnopq";
     

     char convert (char v) {
          v = ((v < 43 || v > 122) ? 0 : (int) cd64[ v - 43 ]);
	  if( v != 0 ) {
		v = ((v == (int)'$') ? 0 : v - 61);
	  } 
	  return (unsigned char) (v - 1);

     }


     void decodeblock( unsigned char *in, unsigned char *out )
     {   
         out[ 0 ] = (unsigned char ) (in[0] << 2 | in[1] >> 4);
         out[ 1 ] = (unsigned char ) (in[1] << 4 | in[2] >> 2);
         out[ 2 ] = (unsigned char ) (((in[2] << 6) & 0xc0) | in[3]);
     }


     int pos=0;
     int i = 0;
     int len = strlen(data);
     char iblock[4];
     char oblock[3];
     while (i < len) {
          iblock[0] = convert( data[i] );
	  iblock[1] = convert( data[i+1] );
	  iblock[2] = convert( data[i+2] );
	  iblock[3] = convert( data[i+3] );
	  decodeblock(iblock,oblock);
	  
	  /*
	  puts(iblock);
	  puts(oblock);
	  printf("position %i ",pos);
	  */
	  
	  result[pos] = oblock[0];
	  pos += 1;
	  if (data[i+2] != '='){ 
	       result[pos] = oblock[1];
	       pos += 1;
	  }
	  if (data[i+3] != '='){
	       result[pos] = oblock[2];
	       pos += 1;
	  }     
	  i +=4;
     }
     /*result[pos] = 0;*/
     return pos;

}

